#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.distance_unit import DistanceUnit
from ..enum.user_role import UserRole
from ..enum.user_status import UserStatus


@dataclass(frozen=True)
class EditUserInput(DataClassJsonMixin):
    id: str
    distanceUnit: Optional[DistanceUnit] = None
    firstName: Optional[str] = None
    lastName: Optional[str] = None
    organizationFk: Optional[str] = None
    role: Optional[UserRole] = None
    status: Optional[UserStatus] = None
