#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config


@dataclass(frozen=True)
class PropertyInput(DataClassJsonMixin):
    propertyTypeID: str
    booleanValue: Optional[bool] = None
    floatValue: Optional[Number] = None
    id: Optional[str] = None
    intValue: Optional[int] = None
    isEditable: Optional[bool] = None
    isInstanceProperty: Optional[bool] = None
    latitudeValue: Optional[Number] = None
    longitudeValue: Optional[Number] = None
    nodeIDValue: Optional[str] = None
    propertyTypeValueID: Optional[str] = None
    rangeFromValue: Optional[Number] = None
    rangeToValue: Optional[Number] = None
    stringValue: Optional[str] = None
