#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config


@dataclass(frozen=True)
class EditRecommendationsInput(DataClassJsonMixin):
    alarmType: str
    externalID: str
    id: str
    longDescription: str
    priority: int
    recommendationsCategory: str
    recommendationsSources: str
    resource: str
    shortDescription: str
    status: bool
    vendor: str
    command: Optional[str] = None
    runbook: Optional[str] = None
    used: Optional[int] = None
    userApprobed: Optional[str] = None
