#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from .service_endpoint_definition import ServiceEndpointDefinitionFragment, QUERY as ServiceEndpointDefinitionFragmentQuery

# fmt: off
QUERY: List[str] = ServiceEndpointDefinitionFragmentQuery + ["""
fragment ServiceEndpointFragment on ServiceEndpoint {
  id
  definition {
    ...ServiceEndpointDefinitionFragment
  }
  equipment {
    id
  }
}

"""]

@dataclass(frozen=True)
class ServiceEndpointFragment(DataClassJsonMixin):
    @dataclass(frozen=True)
    class ServiceEndpointDefinition(ServiceEndpointDefinitionFragment):
        pass

    @dataclass(frozen=True)
    class Equipment(DataClassJsonMixin):
        id: str

    id: str
    definition: ServiceEndpointDefinition
    equipment: Equipment
