#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from ..input.property_input import PropertyInput


@dataclass(frozen=True)
class AddEquipmentInput(DataClassJsonMixin):
    name: str
    properties: List[PropertyInput]
    type: str
    externalId: Optional[str] = None
    location: Optional[str] = None
    parent: Optional[str] = None
    positionDefinition: Optional[str] = None
    workOrder: Optional[str] = None
