#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class LifeCycleStatus(Enum):
    INSTALLING = "INSTALLING"
    OPERATING = "OPERATING"
    PLANNING = "PLANNING"
    RETIRED = "RETIRED"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "LifeCycleStatus":
        return cls.MISSING_ENUM
