#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.file_type import FileType


@dataclass(frozen=True)
class FileInput(DataClassJsonMixin):
    fileName: str
    storeKey: str
    annotation: Optional[str] = None
    fileType: Optional[FileType] = None
    id: Optional[str] = None
    mimeType: Optional[str] = None
    modificationTime: Optional[int] = None
    sizeInBytes: Optional[int] = None
    uploadTime: Optional[int] = None
