#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

SYMPHONY_URI = "https://{}"
LOCALHOST_SERVER = "localtest.me"
LOCALHOST_SYMPHONY = f"https://{{}}.{LOCALHOST_SERVER}"
SYMPHONY_GRAPHQL = "/apollo"
SYMPHONY_STORE_PUT = "/store/put"
SYMPHONY_LOGIN = "/user/login"
SYMPHONY_STORE_DELETE = "/store/delete?key={}"
