#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class WorkOrderStatus(Enum):
    BLOCKED = "BLOCKED"
    CANCELED = "CANCELED"
    CLOSED = "CLOSED"
    DONE = "DONE"
    IN_PROGRESS = "IN_PROGRESS"
    PENDING = "PENDING"
    PLANNED = "PLANNED"
    SUBMITTED = "SUBMITTED"
    SUSPENDED = "SUSPENDED"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "WorkOrderStatus":
        return cls.MISSING_ENUM
